USER_AGENT = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.15; rv:103.0) Gecko/20100101 Firefox/103.0"

PROBLEM_DIFFICULTY = [
    ("暂未评定", "black"),
    ("入门", "red"),
    ("普及-", "yellow"),
    ("普及/提高-", "yellow"),
    ("普及+/提高", "green"),
    ("提高+/省选-", "blue"),
    ("省选/NOI-", "magenta"),
    ("NOI/NOI+/CTSC", "black-bright"),
]
